#!/usr/bin/env python

import sys
import time
import os
from python_sdk_api.sxd_api import *


def check_sdk_rc(rc):
    if rc != SXD_STATUS_SUCCESS:
        print "bad sdk rc %d" % rc
        exit(1)


def usage():
    print "For write ./sxd_api_i2c.py w log_port i2c_addr dev_addr data"
    print "Example   ./sxd_api_i2c.py w   2        0x40   0x1  0x1234"
    print "For read ./sxd_api_i2c.py r log_port i2c_addr dev_addr"
    print "Example  ./sxd_api_i2c.py r   2        0x40   0x1"


def sxd_i2c_reg_write(port, i2c_dev_address, reg, data):

    print "Writing Data to Port %d, i2c Address 0x%02X, Device Address 0x%02X, Data 0x%04X\n" % (port, i2c_dev_address, reg, data)

    mcia = ku_mcia_reg()

    meta = sxd_reg_meta_t()
    meta.dev_id = 1
    meta.swid = 0
    meta.access_cmd = SXD_ACCESS_CMD_SET

    mcia.module = port - 1
    mcia.page_number = 0
    mcia.i2c_device_address = i2c_dev_address
    mcia.device_address = reg
    mcia.size = 2
    mcia.dword_0 = (data << 16) & 0xFFFF0000

    rc = sxd_access_reg_mcia(mcia, meta, 1, None, None)
    check_sdk_rc(rc)


def sxd_i2c_reg_read(port, i2c_dev_address, reg):

    print "Reading Data from Port %d, i2c Address 0x%02X, Device Address 0x%02X\n" % (port, i2c_dev_address, reg)

    mcia = ku_mcia_reg()
    meta = sxd_reg_meta_t()
    meta.dev_id = 1
    meta.swid = 0
    meta.access_cmd = SXD_ACCESS_CMD_GET

    mcia.page_number = 0
    mcia.module = port - 1
    mcia.i2c_device_address = i2c_dev_address
    mcia.device_address = reg
    mcia.size = 2

    rc = sxd_access_reg_mcia(mcia, meta, 1, None, None)
    check_sdk_rc(rc)
    print "[+] dword0 : 0x%08x" % (mcia.dword_0)


def main():

    print "[+] initializing register access"
    sxd_access_reg_init(0, None, 4)

    if len(sys.argv) is 6:
        cmd_str = str(sys.argv[1])
        if cmd_str != "w":
            usage()
            exit(0)

        if 'x' or 'X' in str(sys.argv[2]):
            log_port = int(str(sys.argv[2]), 16)
        else:
            log_port = int(str(sys.argv[2]), 10)

        if log_port <= 0:
            usage()
            exit(0)

        i2c = int(str(sys.argv[3]), 16)
        reg = int(str(sys.argv[4]), 16)
        data = int(str(sys.argv[5]), 16)
        sxd_i2c_reg_write(log_port, i2c, reg, data)

    elif len(sys.argv) is 5:
        cmd_str = str(sys.argv[1])
        if cmd_str != "r":
            usage()
            exit(0)

        if 'x' or 'X' in str(sys.argv[2]):
            log_port = int(str(sys.argv[2]), 16)
        else:
            log_port = int(str(sys.argv[2]), 10)

        if log_port <= 0:
            usage()
            exit(0)

        i2c = int(str(sys.argv[3]), 16)
        reg = int(str(sys.argv[4]), 16)
        sxd_i2c_reg_read(log_port, i2c, reg)

    else:
        usage()
        exit(0)


if __name__ == "__main__":
    main()
